"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSettings = exports.insertAndGetId = exports.query = exports.init = void 0;
/* eslint no-console: 0 */
const knex_1 = require("knex");
const path = require("node:path");
// eslint-disable-next-line @typescript-eslint/no-var-requires
require('dotenv').config();
let settings = null;
const db = (0, knex_1.knex)(getSettings());
async function init() {
    // eslint-disable-next-line no-console
    console.log('Running database migrations');
    await db.migrate.latest();
}
exports.init = init;
exports.default = db;
/**
 * A shortcut for easily executing select queries.
 *
 * Use of this should be phased out, but it helped with the migration from
 * before knex.
 */
async function query(query, params = []) {
    // Knex returns weird typings for the raw function,
    const result = (await db.raw(query, params));
    switch (settings?.client) {
        case 'pg':
            return result.rows;
        case 'sqlite3':
            return result;
        case 'mysql2':
            return result[0];
        default:
            throw new Error(`Unknown driver: ${settings?.client}`);
    }
}
exports.query = query;
/**
 * Inserts a single record and return id
 *
 * Sadly we need a helper function.
 */
async function insertAndGetId(tableName, data) {
    const result = await db(tableName).insert(data, 'id');
    return result[0]?.id ?? result[0];
}
exports.insertAndGetId = insertAndGetId;
function getSettings() {
    let connection;
    let client;
    let searchPath;
    if (process.env.PG_DATABASE) {
        console.warn('The PG_* environment variables are deprecated. Use DB_DRIVER=pg, DB_HOST, DB_USER, DB_PASSWORD, DB_DATABASE instead');
        client = 'pg';
        connection = {
            host: process.env.PG_HOST || '127.0.0.1',
            port: parseInt(process.env.PG_PORT, 10) || 5432,
            user: process.env.PG_USER,
            password: process.env.PG_PASSWORD,
            database: process.env.PG_DATABASE,
        };
        searchPath = [
            connection.user,
            connection.database,
        ];
    }
    else if (process.env.MYSQL_DATABASE) {
        console.warn('The MYSQL_* environment variables are deprecated. Use DB_DRIVER=mysql, DB_HOST, DB_USER, DB_PASSWORD, DB_DATABASE instead');
        client = 'mysql2';
        connection = {
            host: process.env.MYSQL_HOST || '127.0.0.1',
            port: parseInt(process.env.MYSQL_PORT, 10) || 3306,
            user: process.env.MYSQL_USER,
            password: process.env.MYSQL_PASSWORD,
            database: process.env.MYSQL_DATABASE,
        };
        if (process.env.MYSQL_INSTANCE_CONNECTION_NAME) {
            connection.socketPath = '/cloudsql/' + process.env.MYSQL_INSTANCE_CONNECTION_NAME;
            delete connection.host;
            delete connection.port;
        }
    }
    else {
        switch (process.env.DB_DRIVER) {
            case 'pg':
                client = 'pg';
                connection = {
                    host: process.env.DB_HOST || '127.0.0.1',
                    port: parseInt(process.env.DB_PORT, 10) || 5432,
                    user: process.env.DB_USER,
                    password: process.env.DB_PASSWORD,
                    database: process.env.DB_DATABASE,
                };
                searchPath = [
                    connection.user,
                    connection.database,
                ];
                break;
            case 'mysql':
            case 'mysql2':
                client = 'mysql2';
                connection = {
                    host: process.env.DB_HOST || '127.0.0.1',
                    port: parseInt(process.env.DB_PORT, 10) || 3306,
                    user: process.env.DB_USER,
                    password: process.env.DB_PASSWORD,
                    database: process.env.DB_DATABASE,
                };
                if (process.env.MYSQL_INSTANCE_CONNECTION_NAME) {
                    connection.socketPath = '/cloudsql/' + process.env.MYSQL_INSTANCE_CONNECTION_NAME;
                }
                else {
                    delete connection.host;
                    delete connection.port;
                }
                break;
            case 'sqlite':
            case 'sqlite3':
            case undefined:
                client = 'sqlite3';
                if (process.env.DB_DRIVER === undefined) {
                    console.warn('No database settings were found, so we\'re creating a sqlite database in the current directory. This is not recommended for production');
                }
                connection = {
                    filename: process.env.DB_FILENAME || 'a12nserver.sqlite3'
                };
                break;
            default:
                throw new Error(`Unknown value for DB_DRIVER: ${process.env.DB_DRIVER}`);
        }
    }
    settings = {
        client,
        connection,
        searchPath,
        migrations: {
            directory: path.join(__dirname, 'migrations'),
            loadExtensions: ['.js'],
        },
        pool: { min: 0, max: 10 },
        debug: process.env.DEBUG ? true : false,
    };
    return settings;
}
exports.getSettings = getSettings;
//# sourceMappingURL=database.js.map