"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const controller_1 = require("@curveball/controller");
const server_1 = require("@simplewebauthn/server");
const webAuthnService = require("../service");
const server_settings_1 = require("../../../server-settings");
class WebAuthnAttestationController extends controller_1.default {
    async get(ctx) {
        const user = ctx.session.registerUser;
        const registrationOptions = (0, server_1.generateRegistrationOptions)({
            rpName: (0, server_settings_1.getSetting)('webauthn.serviceName'),
            rpID: (0, server_settings_1.getSetting)('webauthn.relyingPartyId') || new URL(process.env.PUBLIC_URI).host,
            userID: user.id.toString(),
            userName: user.nickname,
            timeout: 60000,
            attestationType: 'indirect',
            excludeCredentials: (await webAuthnService.findDevicesByUser(user)).map(device => ({
                id: Buffer.from(device.credentialID),
                type: 'public-key',
            })),
            /**
             * The optional authenticatorSelection property allows for specifying more constraints around
             * the types of authenticators that users to can use for registration
             */
            authenticatorSelection: {
                authenticatorAttachment: 'cross-platform',
                userVerification: 'preferred',
                requireResidentKey: false,
            },
        });
        ctx.session.webAuthnChallengeRegister = registrationOptions.challenge;
        ctx.response.body = registrationOptions;
    }
    async post(ctx) {
        const user = ctx.session.registerUser;
        const body = ctx.request.body;
        const expectedChallenge = ctx.session.webAuthnChallengeRegister;
        ctx.session.webAuthnChallengeRegister = null;
        let verification;
        try {
            verification = await (0, server_1.verifyRegistrationResponse)({
                credential: body,
                expectedChallenge,
                expectedOrigin: (0, server_settings_1.getSetting)('webauthn.expectedOrigin') || new URL(process.env.PUBLIC_URI).origin,
                expectedRPID: (0, server_settings_1.getSetting)('webauthn.relyingPartyId') || new URL(process.env.PUBLIC_URI).host,
            });
        }
        catch (error) {
            /* eslint-disable-next-line no-console */
            console.error(error);
            ctx.status = 400;
            ctx.response.body = { error: error.message };
            return;
        }
        const { verified, registrationInfo } = verification;
        if (verified) {
            const { credentialPublicKey, credentialID, counter } = registrationInfo;
            const existingDevice = (await webAuthnService.findDevicesByUser(user)).find(device => device.credentialID === credentialID);
            if (!existingDevice) {
                await webAuthnService.save({
                    user,
                    credentialID: credentialPublicKey,
                    publicKey: credentialID,
                    counter,
                });
                ctx.session.registerUser = null;
            }
        }
        ctx.response.body = { verified };
    }
}
exports.default = new WebAuthnAttestationController();
//# sourceMappingURL=registration.js.map