"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http_errors_1 = require("@curveball/http-errors");
const oauth2Service = require("./../oauth2/service");
const whitelistPath = [
    '/login',
    '/assets',
    '/health',
    '/register',
    '/authorize',
    '/reset-password',
    '/token',
    '/introspect',
    '/revoke',
    '/.well-known/jwks.json',
    '/.well-known/oauth-authorization-server',
];
class AuthHelper {
    constructor(principal) {
        this.principal = principal;
    }
    /**
     * Returns true if there's a logged in user (or app)
     */
    isLoggedIn() {
        return this.principal !== null;
    }
    /**
     * Returns true if the logged in user matches the passsed-in principal
     */
    equals(principal) {
        return this.principal?.id === principal.id;
    }
}
/**
 * The login middleware ensures that the current HTTP request is authenticated
 */
function default_1() {
    return async (ctx, next) => {
        let inWhitelist = false;
        for (const path of whitelistPath) {
            if (ctx.path === path || ctx.path.startsWith(path + '/')) {
                inWhitelist = true;
                break;
            }
        }
        if (!inWhitelist && ctx.request.headers.has('Authorization')) {
            // We had an authorization header, lets validate it
            const authHeader = ctx.request.headers.get('Authorization');
            const [authType, accessToken] = authHeader.split(' ');
            if (authType.toLowerCase() !== 'bearer') {
                throw new http_errors_1.Unauthorized('Only Bearer authentication is currently supported', 'Bearer');
            }
            let token;
            try {
                token = await oauth2Service.getTokenByAccessToken(accessToken);
            }
            catch (e) {
                if (e instanceof http_errors_1.NotFound) {
                    throw new http_errors_1.Unauthorized('Bearer token not recognized');
                }
                else {
                    throw e;
                }
            }
            // We are logged in!
            ctx.auth = new AuthHelper(token.user);
            return next();
        }
        ctx.auth = new AuthHelper(ctx.session.user || null);
        if (ctx.session.user) {
            // The user was logged in via a session cookie.
            return next();
        }
        if (inWhitelist) {
            return next();
        }
        // Not logged in.
        ctx.status = 303;
        ctx.response.headers.set('Location', '/login');
    };
}
exports.default = default_1;
//# sourceMappingURL=login.js.map