"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const controller_1 = require("@curveball/controller");
const privilegeService = require("../../privilege/service");
const hal = require("../formats/hal");
const http_errors_1 = require("@curveball/http-errors");
const service_1 = require("../service");
const principalService = require("../../principal/service");
const bcrypt = require("bcrypt");
const crypto_1 = require("../../crypto");
class ClientCollectionController extends controller_1.default {
    async get(ctx) {
        const app = await principalService.findByExternalId(ctx.params.id, 'app');
        if (ctx.auth.equals(app)) {
            if (!await privilegeService.hasPrivilege(ctx, 'admin')) {
                throw new http_errors_1.Forbidden('Only users with the "admin" privilege can inspect OAuth2 clients that are not your own');
            }
        }
        const clients = await (0, service_1.findByApp)(app);
        ctx.response.body = hal.collection(app, clients);
    }
    async post(ctx) {
        const app = await principalService.findByExternalId(ctx.params.id, 'app');
        if (!await privilegeService.hasPrivilege(ctx, 'admin')) {
            throw new http_errors_1.Forbidden('Only users with the "admin" privilege can inspect OAuth2 clients that are not your own');
        }
        const allowedGrantTypes = [];
        if (ctx.request.body.allowClientCredentials) {
            allowedGrantTypes.push('client_credentials');
        }
        if (ctx.request.body.allowAuthorizationCode) {
            allowedGrantTypes.push('authorization_code');
        }
        if (ctx.request.body.allowImplicit) {
            allowedGrantTypes.push('implicit');
        }
        if (ctx.request.body.allowRefreshToken) {
            allowedGrantTypes.push('refresh_token');
        }
        if (ctx.request.body.allowPassword) {
            allowedGrantTypes.push('password');
        }
        let clientId = ctx.request.body.clientId;
        const redirectUris = ctx.request.body.redirectUris.trim().split(/\r\n|\n/).filter((line) => !!line);
        if (!clientId) {
            clientId = await (0, crypto_1.generatePublicId)();
        }
        else if (clientId.length < 6) {
            throw new http_errors_1.UnprocessableEntity('clientId must be at least 6 characters or left empty');
        }
        if (!allowedGrantTypes) {
            throw new http_errors_1.UnprocessableEntity('You must specify the allowedGrantTypes property');
        }
        const clientSecret = `secret-token:${await (0, crypto_1.generateSecretToken)()}`;
        const newClient = {
            clientId,
            app,
            allowedGrantTypes: allowedGrantTypes,
            clientSecret: await bcrypt.hash(clientSecret, 12),
            requirePkce: ctx.request.body.requirePkce ?? false,
        };
        const client = await (0, service_1.create)(newClient, redirectUris);
        ctx.response.body = hal.newClientSuccess(client, redirectUris, clientSecret);
    }
}
exports.default = new ClientCollectionController();
//# sourceMappingURL=collection.js.map