"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateSecret = exports.create = exports.getOAuth2ClientFromBody = exports.getOAuth2ClientFromBasicAuth = exports.findByApp = exports.findById = exports.findByClientId = void 0;
const bcrypt = require("bcrypt");
const principalService = require("../principal/service");
const database_1 = require("../database");
const http_errors_1 = require("@curveball/http-errors");
const errors_1 = require("../oauth2/errors");
const parse_basic_auth_1 = require("./parse-basic-auth");
async function findByClientId(clientId) {
    const result = await (0, database_1.default)('oauth2_clients')
        .select('*')
        .where('client_id', clientId);
    if (!result.length) {
        throw new http_errors_1.NotFound('OAuth2 client_id not recognized');
    }
    const record = result[0];
    const app = await principalService.findById(record.user_id, 'app');
    if (!app.active) {
        throw new Error(`App ${app.nickname} is not active`);
    }
    return mapRecordToModel(record, app);
}
exports.findByClientId = findByClientId;
async function findById(id) {
    const result = await (0, database_1.default)('oauth2_clients')
        .select('*')
        .where('id', id);
    if (!result.length) {
        throw new http_errors_1.NotFound('OAuth2 id not recognized');
    }
    const record = result[0];
    const app = await principalService.findById(record.user_id, 'app');
    if (!app.active) {
        throw new Error(`App ${app.nickname} is not active`);
    }
    return mapRecordToModel(record, app);
}
exports.findById = findById;
async function findByApp(app) {
    const result = await (0, database_1.default)('oauth2_clients')
        .select('*')
        .where('user_id', app.id);
    return result.map((record) => mapRecordToModel(record, app));
}
exports.findByApp = findByApp;
function mapRecordToModel(record, app) {
    return {
        id: record.id,
        clientId: record.client_id,
        clientSecret: record.client_secret,
        app,
        allowedGrantTypes: record.allowed_grant_types.split(' '),
        requirePkce: !!record.require_pkce,
    };
}
async function getOAuth2ClientFromBasicAuth(ctx) {
    let oauth2Client;
    const basicAuth = (0, parse_basic_auth_1.default)(ctx);
    if (!basicAuth) {
        throw new http_errors_1.Unauthorized('Invalid Basic Auth header');
    }
    try {
        oauth2Client = await findByClientId(basicAuth[0]);
    }
    catch (e) {
        if (e instanceof http_errors_1.NotFound) {
            throw new http_errors_1.Unauthorized('Client id or secret incorrect', 'Basic');
        }
        else {
            // Rethrow
            throw e;
        }
    }
    if (!await validateSecret(oauth2Client, basicAuth[1])) {
        throw new http_errors_1.Unauthorized('Client id or secret incorrect', 'Basic');
    }
    return oauth2Client;
}
exports.getOAuth2ClientFromBasicAuth = getOAuth2ClientFromBasicAuth;
async function getOAuth2ClientFromBody(ctx) {
    if (!ctx.request.body.client_id) {
        throw new errors_1.InvalidRequest('The "client_id" property is required');
    }
    try {
        return await findByClientId(ctx.request.body.client_id);
    }
    catch (e) {
        if (e instanceof http_errors_1.NotFound) {
            throw new http_errors_1.Unauthorized('Client id unknown', 'Basic');
        }
        else {
            // Rethrow
            throw e;
        }
    }
}
exports.getOAuth2ClientFromBody = getOAuth2ClientFromBody;
async function create(client, redirectUris) {
    const params = {
        client_id: client.clientId,
        client_secret: client.clientSecret,
        user_id: client.app.id,
        allowed_grant_types: client.allowedGrantTypes.join(' '),
        require_pkce: client.requirePkce ? 0 : 1,
    };
    const result = await (0, database_1.insertAndGetId)('oauth2_clients', params);
    for (const uri of redirectUris) {
        await (0, database_1.default)('oauth2_redirect_uris').insert({ oauth2_clients_id: result, uri });
    }
    return {
        id: result,
        ...client,
    };
}
exports.create = create;
async function validateSecret(oauth2Client, secret) {
    return await bcrypt.compare(secret, oauth2Client.clientSecret);
}
exports.validateSecret = validateSecret;
//# sourceMappingURL=service.js.map