"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const controller_1 = require("@curveball/controller");
const http_errors_1 = require("@curveball/http-errors");
const querystring = require("querystring");
const errors_1 = require("../errors");
const oauth2Service = require("../service");
const service_1 = require("../../log/service");
const types_1 = require("../../log/types");
const service_2 = require("../../oauth2-client/service");
class AuthorizeController extends controller_1.default {
    async get(ctx) {
        ctx.response.type = 'text/html';
        let oauth2Client;
        let codeChallengeMethod = undefined;
        if (!['token', 'code'].includes(ctx.query.response_type)) {
            throw new errors_1.InvalidRequest('The "response_type" parameter must be provided, and must be "token" or "code"');
        }
        if (!ctx.query.client_id) {
            throw new errors_1.InvalidRequest('The "client_id" parameter must be provided');
        }
        const clientId = ctx.query.client_id;
        try {
            oauth2Client = await (0, service_2.findByClientId)(clientId);
        }
        catch (e) {
            if (e instanceof http_errors_1.NotFound) {
                throw new errors_1.InvalidClient('Client id incorrect');
            }
            else {
                // Rethrow
                throw e;
            }
        }
        if (!ctx.query.redirect_uri) {
            throw new errors_1.InvalidRequest('The "redirect_uri" parameter must be provided');
        }
        if (oauth2Client.requirePkce && !ctx.query.code_challenge) {
            throw new errors_1.InvalidRequest('This endpoint requires that OAuth2 client support PKCE, and your client did not pass the correct parameters. Either turn off the PKCE requirement for this OAuth2 client, or upgrade to an OAuth2 client library that supports PKCE.');
        }
        if (ctx.query.response_type === 'code') {
            if (!ctx.query.code_challenge && ctx.query.code_challenge_method) {
                throw new errors_1.InvalidRequest('The "code_challenge" must be provided');
            }
            if (ctx.query.code_challenge_method) {
                switch (ctx.query.code_challenge_method) {
                    case 'S256':
                    case 'plain':
                        codeChallengeMethod = ctx.query.code_challenge_method;
                        break;
                    default:
                        throw new errors_1.InvalidRequest('The "code_challenge_method" must be "plain" or "S256"');
                }
            }
            else {
                codeChallengeMethod = ctx.query.code_challenge ? 'plain' : undefined;
            }
        }
        const state = ctx.query.state;
        // const scope = ctx.query.scope;
        const responseType = ctx.query.response_type;
        const redirectUri = ctx.query.redirect_uri;
        const codeChallenge = ctx.query.code_challenge;
        const grantType = responseType === 'code' ? 'authorization_code' : 'implicit';
        if (!oauth2Client.allowedGrantTypes.includes(grantType)) {
            throw new errors_1.UnsupportedGrantType('The current client is not allowed to use the ' + grantType + ' grant_type');
        }
        try {
            await oauth2Service.requireRedirectUri(oauth2Client, redirectUri);
        }
        catch (err) {
            (0, service_1.default)(types_1.EventType.oauth2BadRedirect, ctx);
            throw err;
        }
        if (ctx.session.user !== undefined) {
            if (responseType === 'token') {
                return this.tokenRedirect(ctx, oauth2Client, redirectUri, state);
            }
            else {
                return this.codeRedirect(ctx, oauth2Client, redirectUri, state, codeChallenge, codeChallengeMethod);
            }
        }
        else {
            return this.redirectToLogin(ctx, { 'continue': ctx.request.requestTarget });
        }
    }
    async tokenRedirect(ctx, oauth2Client, redirectUri, state) {
        const token = await oauth2Service.generateTokenForUser(oauth2Client, ctx.session.user);
        ctx.status = 302;
        ctx.response.headers.set('Cache-Control', 'no-cache');
        ctx.response.headers.set('Location', redirectUri + '#' + querystring.stringify({
            access_token: token.accessToken,
            token_type: token.tokenType,
            expires_in: token.accessTokenExpires - Math.round(Date.now() / 1000),
            state: state
        }));
    }
    async codeRedirect(ctx, oauth2Client, redirectUri, state, codeChallenge, codeChallengeMethod) {
        const code = await oauth2Service.generateCodeForUser(oauth2Client, ctx.session.user, codeChallenge, codeChallengeMethod, ctx.sessionId);
        ctx.status = 302;
        ctx.response.headers.set('Cache-Control', 'no-cache');
        ctx.response.headers.set('Location', redirectUri + '?' + querystring.stringify({
            code: code.code,
            state: state
        }));
    }
    /**
     * Redirects to login screen, if login failed
     */
    async redirectToLogin(ctx, params) {
        ctx.response.status = 302;
        ctx.response.headers.set('Location', '/login?' + querystring.stringify(params));
    }
}
exports.default = new AuthorizeController();
//# sourceMappingURL=authorize.js.map