"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPathName = exports.isIdentityValid = exports.recordToModel = exports.save = exports.findByHref = exports.findByIdentity = exports.hasPrincipals = exports.findByExternalId = exports.findById = exports.getPrincipalStats = exports.findAll = exports.fieldNames = exports.InactivePrincipal = void 0;
const http_errors_1 = require("@curveball/http-errors");
const database_1 = require("../database");
const crypto_1 = require("../crypto");
class InactivePrincipal extends Error {
}
exports.InactivePrincipal = InactivePrincipal;
exports.fieldNames = [
    'id',
    'identity',
    'external_id',
    'nickname',
    'created_at',
    'modified_at',
    'type',
    'active'
];
async function findAll(type) {
    const filters = {};
    if (type) {
        filters.type = userTypeToInt(type);
    }
    const result = await (0, database_1.default)('principals')
        .where(filters);
    const principals = [];
    for (const principal of result) {
        principals.push(recordToModel(principal));
    }
    return principals;
}
exports.findAll = findAll;
async function getPrincipalStats() {
    const result = await (0, database_1.default)('principals')
        .select(['type', database_1.default.raw('COUNT(*) as total')])
        .groupBy('type');
    const principalStats = {
        user: 0,
        app: 0,
        group: 0
    };
    for (const principal of result) {
        principalStats[userTypeIntToUserType(principal.type)] = principal.total;
    }
    return principalStats;
}
exports.getPrincipalStats = getPrincipalStats;
async function findById(id, type) {
    const result = await (0, database_1.default)('principals')
        .select(exports.fieldNames)
        .where({ id });
    if (result.length !== 1) {
        throw new http_errors_1.NotFound(`Principal with id: ${id} not found`);
    }
    const principal = recordToModel(result[0]);
    if (type && principal.type !== type) {
        throw new http_errors_1.NotFound(`Principal with id ${id} does not have type ${type}`);
    }
    return principal;
}
exports.findById = findById;
async function findByExternalId(externalId, type) {
    let result = await (0, database_1.default)('principals')
        .select(exports.fieldNames)
        .where({ external_id: externalId });
    if (result.length !== 1 && +(externalId) > 0) {
        // Trying to find the principal but now use the id field
        result = await (0, database_1.default)('principals')
            .select(exports.fieldNames)
            .where({ id: +externalId });
        if (result.length !== 1) {
            throw new http_errors_1.NotFound(`Principal with id: ${externalId} not found`);
        }
    }
    const principal = recordToModel(result[0]);
    if (type && principal.type !== type) {
        throw new http_errors_1.NotFound(`Principal with id ${externalId} does not have type ${type}`);
    }
    return principal;
}
exports.findByExternalId = findByExternalId;
/**
 * Returns true if more than 1 principal exists in the system.
 *
 * If there are 0 principals, this puts a12nserver in setup mode, which allows a
 * person to create the first user in the system, automatically activate it
 * and make them an admin.
 */
async function hasPrincipals() {
    const result = await (0, database_1.query)('SELECT 1 FROM principals LIMIT 1');
    return result.length > 0;
}
exports.hasPrincipals = hasPrincipals;
async function findByIdentity(identity) {
    const result = await (0, database_1.query)(`SELECT ${exports.fieldNames.join(', ')} FROM principals WHERE identity = ?`, [identity]);
    if (result.length !== 1) {
        throw new http_errors_1.NotFound(`Principal with identity: ${identity} not found`);
    }
    return recordToModel(result[0]);
}
exports.findByIdentity = findByIdentity;
/**
 * Finds a principal by its href'.
 *
 * This can be a string like /user/1, or a full url.
 * It can also be the uri listed in the 'identity' field.
 */
async function findByHref(href) {
    const pathName = getPathName(href);
    const matches = pathName.match(/^\/(user|app|group)\/([0-9a-zA-Z_-]+)$/);
    if (!matches) {
        return findByIdentity(href);
    }
    let typeFilter;
    switch (matches[1]) {
        case 'user':
            // Backwards compatibility
            typeFilter = [1, 2, 3];
            break;
        case 'app':
            typeFilter = [2];
            break;
        case 'group':
            // Backwards compatibility
            typeFilter = [3];
            break;
    }
    const result = await (0, database_1.query)(`SELECT ${exports.fieldNames.join(', ')} FROM principals WHERE type IN (${typeFilter.map(_ => '?').join(',')}) AND id = ?`, [...typeFilter, matches[2]]);
    if (result.length !== 1) {
        throw new http_errors_1.NotFound('Principal with href: ' + href + ' not found');
    }
    return recordToModel(result[0]);
}
exports.findByHref = findByHref;
async function save(principal) {
    if (!isExistingPrincipal(principal)) {
        const externalId = await (0, crypto_1.generatePublicId)();
        const newPrincipalRecord = {
            identity: principal.identity,
            external_id: externalId,
            nickname: principal.nickname,
            type: userTypeToInt(principal.type),
            active: principal.active ? 1 : 0,
            modified_at: Date.now(),
            created_at: Date.now(),
        };
        const result = await (0, database_1.insertAndGetId)('principals', newPrincipalRecord);
        return {
            id: result,
            href: `/${principal.type}/${result}`,
            externalId,
            ...principal
        };
    }
    else {
        // Update user
        if (!isIdentityValid(principal.identity)) {
            throw new http_errors_1.UnprocessableEntity('Identity must be a valid URI');
        }
        principal.modifiedAt = new Date();
        const updatePrincipalRecord = {
            identity: principal.identity,
            nickname: principal.nickname,
            active: principal.active ? 1 : 0,
            modified_at: principal.modifiedAt.getTime(),
        };
        await (0, database_1.default)('principals')
            .where('id', principal.id)
            .update(updatePrincipalRecord);
        return principal;
    }
}
exports.save = save;
function userTypeIntToUserType(input) {
    switch (input) {
        case 1: return 'user';
        case 2: return 'app';
        case 3: return 'group';
        default:
            throw new Error('Unknown user type id: ' + input);
    }
}
function userTypeToInt(input) {
    switch (input) {
        case 'user': return 1;
        case 'app': return 2;
        case 'group': return 3;
    }
}
function recordToModel(user) {
    return {
        id: user.id,
        href: `/${userTypeIntToUserType(user.type)}/${user.external_id}`,
        identity: user.identity,
        externalId: user.external_id,
        nickname: user.nickname,
        createdAt: new Date(user.created_at),
        modifiedAt: new Date(user.modified_at),
        type: userTypeIntToUserType(user.type),
        active: !!user.active
    };
}
exports.recordToModel = recordToModel;
function isIdentityValid(identity) {
    const regex = /^(?:[A-Za-z]+:\S*$)?/;
    return regex.test(identity);
}
exports.isIdentityValid = isIdentityValid;
function isExistingPrincipal(user) {
    return user.id !== undefined;
}
function getPathName(href) {
    let url;
    try {
        url = new URL(href);
    }
    catch {
        return href;
    }
    return url.pathname;
}
exports.getPathName = getPathName;
//# sourceMappingURL=service.js.map