"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addPrivilegeForUser = exports.replacePrivilegeForUser = exports.findPrivilege = exports.findPrivileges = exports.hasPrivilege = exports.getImmediatePrivilegesForPrincipal = exports.getPrivilegesForPrincipal = void 0;
const database_1 = require("../database");
async function getPrivilegesForPrincipal(principal) {
    const recursiveGroupIds = await getRecursiveGroupIds(principal.id);
    const result = await (0, database_1.query)(`SELECT resource, privilege FROM user_privileges WHERE user_id IN (${recursiveGroupIds.map(_ => '?').join(',')})`, [...recursiveGroupIds]);
    return result.reduce((currentPrivileges, row) => {
        const privileges = Object.assign({}, currentPrivileges);
        // eslint-disable-next-line no-prototype-builtins
        if (privileges.hasOwnProperty(row.resource)) {
            if (privileges[row.resource].indexOf(row.privilege) === -1) {
                privileges[row.resource].push(row.privilege);
            }
        }
        else {
            privileges[row.resource] = [row.privilege];
        }
        return privileges;
    }, {});
}
exports.getPrivilegesForPrincipal = getPrivilegesForPrincipal;
async function getImmediatePrivilegesForPrincipal(principal) {
    const result = await (0, database_1.query)('SELECT resource, privilege FROM user_privileges WHERE user_id = ?', [principal.id]);
    return result.reduce((currentPrivileges, row) => {
        const privileges = Object.assign({}, currentPrivileges);
        // eslint-disable-next-line no-prototype-builtins
        if (privileges.hasOwnProperty(row.resource)) {
            if (privileges[row.resource].indexOf(row.privilege) === -1) {
                privileges[row.resource].push(row.privilege);
            }
        }
        else {
            privileges[row.resource] = [row.privilege];
        }
        return privileges;
    }, {});
}
exports.getImmediatePrivilegesForPrincipal = getImmediatePrivilegesForPrincipal;
async function hasPrivilege(who, privilege, resource = '*') {
    let user;
    if (isContext(who)) {
        if (!who.auth.isLoggedIn()) {
            throw new Error('Cannot check privilege for unauthenticated user');
        }
        user = who.auth.principal;
    }
    else {
        user = who;
    }
    const privileges = await getPrivilegesForPrincipal(user);
    return privileges['*']?.includes(privilege) || privileges[resource]?.includes(privilege) || false;
}
exports.hasPrivilege = hasPrivilege;
function isContext(input) {
    return input.request !== undefined && input.response !== undefined;
}
/**
 * Returns the list of 'privilege types'
 */
async function findPrivileges() {
    const result = await (0, database_1.query)(`
  SELECT privileges.privilege, privileges.description FROM privileges
  UNION ALL
  SELECT DISTINCT user_privileges.privilege, null FROM user_privileges
  LEFT JOIN privileges
  ON privileges.privilege = user_privileges.privilege
  WHERE privileges.privilege IS NULL`);
    return result;
}
exports.findPrivileges = findPrivileges;
async function findPrivilege(privilege) {
    const result = await (0, database_1.query)('SELECT privilege, description FROM privileges WHERE privilege = ?', [privilege]);
    if (result.length !== 1) {
        return {
            privilege: privilege,
            description: ''
        };
    }
    return result[0];
}
exports.findPrivilege = findPrivilege;
async function replacePrivilegeForUser(user, privilegeMap) {
    await database_1.default.transaction(async (trx) => {
        await trx.raw('DELETE FROM user_privileges WHERE user_id = ?', [user.id]);
        for (const [resource, privileges] of Object.entries(privilegeMap)) {
            for (const privilege of privileges) {
                await trx('user_privileges').insert({
                    user_id: user.id,
                    privilege,
                    resource,
                });
            }
        }
        await trx.commit();
    });
}
exports.replacePrivilegeForUser = replacePrivilegeForUser;
async function addPrivilegeForUser(user, privilege, resource) {
    await (0, database_1.default)('user_privileges').insert({
        user_id: user.id,
        privilege,
        resource,
    });
}
exports.addPrivilegeForUser = addPrivilegeForUser;
/**
 * Returns a list of all group principals for a user.
 *
 * This basically returns a list of ids of all groups that a user is a part of,
 * recursively, and also includes the id of the user itself.
 */
async function getRecursiveGroupIds(principalId) {
    const q = 'SELECT group_id FROM group_members WHERE user_id = ?';
    const result = (await (0, database_1.query)(q, [principalId]));
    const ids = [
        principalId
    ];
    for (const row of result) {
        ids.push(...await getRecursiveGroupIds(row.group_id));
    }
    return ids;
}
//# sourceMappingURL=service.js.map