"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasTotp = exports.validateTotp = exports.hasPassword = exports.validatePassword = exports.updatePassword = exports.createPassword = void 0;
const bcrypt = require("bcrypt");
const otplib = require("otplib");
const database_1 = require("../database");
async function createPassword(user, password) {
    await (0, database_1.default)('user_passwords').insert({
        user_id: user.id,
        password: await bcrypt.hash(password, 12)
    });
}
exports.createPassword = createPassword;
async function updatePassword(user, password) {
    const query = 'INSERT INTO user_passwords (password, user_id) VALUES (?, ?) ON DUPLICATE KEY UPDATE password = ?';
    const hashedPw = await bcrypt.hash(password, 12);
    await database_1.default.raw(query, [hashedPw, user.id, hashedPw]);
}
exports.updatePassword = updatePassword;
/**
 * Returns true or false if the password was correct.
 *
 * Calling this method multiple times might result in a block.
 */
async function validatePassword(user, password) {
    const result = await (0, database_1.default)('user_passwords')
        .select('password')
        .where('user_id', user.id);
    const hashes = result.map((row) => row.password);
    for (const hash of hashes) {
        if (await bcrypt.compare(password, hash.toString('utf-8'))) {
            return true;
        }
    }
    return false;
}
exports.validatePassword = validatePassword;
async function hasPassword(user) {
    const result = await (0, database_1.default)('user_passwords')
        .select('user_id')
        .where('user_id', user.id);
    return result.length > 0;
}
exports.hasPassword = hasPassword;
/**
 * Returns true or false if the totp token was correct.
 *
 * Calling this method multiple times might result in a block.
 */
async function validateTotp(user, token) {
    const result = await (0, database_1.default)('user_totp')
        .select('secret')
        .where('user_id', user.id);
    if (!result.length) {
        // Not set up
        return false;
    }
    const secret = result[0].secret;
    return otplib.authenticator.check(token, secret);
}
exports.validateTotp = validateTotp;
/**
 * Returns true or false if the totp was provided or not.
 *
 *
 */
async function hasTotp(user) {
    const result = await (0, database_1.default)('user_totp')
        .select('secret')
        .where('user_id', user.id);
    return result.length !== 0;
}
exports.hasTotp = hasTotp;
//# sourceMappingURL=service.js.map